# Generated by Django 4.0.3 on 2022-05-18 08:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_tenants', '0002_remove_tenant_is_main_remove_tenant_slug_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='tenant',
            name='slug',
            field=models.CharField(default='encore', max_length=100),
            preserve_default=False,
        ),
    ]
