# Generated by Django 4.0.3 on 2022-05-14 14:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_tenants', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='tenant',
            name='is_main',
        ),
        migrations.RemoveField(
            model_name='tenant',
            name='slug',
        ),
        migrations.AddField(
            model_name='tenant',
            name='description',
            field=models.TextField(blank=True, null=True),
        ),
    ]
