# Generated by Django 4.0.3 on 2022-05-10 10:56

from django.db import migrations, models
import django_extensions.db.fields
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Tenant',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(default=uuid.uuid4, editable=False, unique=True)),
                ('is_main', models.BooleanField(default=False, editable=False)),
                ('slug', django_extensions.db.fields.AutoSlugField(allow_unicode=True, blank=True, editable=False, max_length=100, populate_from='name', unique=True)),
                ('name', models.CharField(max_length=100)),
            ],
            options={
                'verbose_name': 'tenant',
                'verbose_name_plural': 'tenants',
                'abstract': False,
                'swappable': 'TENANT_MODEL',
            },
        ),
    ]
